import os
import database_connection as dc
import database_query as dq
from configuration import get_configuration_value


# Where we will store the address translation files
def get_translation_folder():
    return "trans"


# Get the data name for an adress translation file corresponding to a data_name
def generate_trans_name(data_name):
    return data_name + "_tr"


def generate_trans_file_name(data_name):
    return generate_trans_name(data_name) + ".atf"


# Get the full path and filename to an address translation file
def get_trans_file_path(data_name):
    return os.path.join(get_translation_folder(), generate_trans_file_name(data_name))


def create_address_registry_file(data_name):
    print("Create an address registry translation file.", flush=True)
    trans_file_path = get_trans_file_path(data_name)
    print("Create an address registry translation file:", trans_file_path, flush=True)
    connection = dc.get_connection(False, trans_file_path)
    print("Execute:", flush=True)
    connection.execute('''CREATE TABLE address
             (id INTEGER PRIMARY KEY AUTOINCREMENT,
             pubkey     CHAR,
             ip         CHAR,
             port       CHAR
              );''')
    print("Commit:", flush=True)
    connection.commit()
    print("Done.", flush=True)


# dict of addresses for this data_name
def get_all_addresses(data_name):
    # todo: flag silent translate in the db for that (-silent) to work

    # get the nodes, having fingers pointing on our data
    data_id = dq.get_link_dataid_by_name(data_name)
    data_addresses = dq.get_address_list_as_dict(data_id)

    # Add addresses pointing to
    translation_data_id = dq.get_link_dataid_by_name(generate_trans_name(data_name))
    if translation_data_id:
        translation_data_addresses = dq.get_address_list_as_dict(translation_data_id)

        # todo: in Python 3.9 or something, you can use a more delicate way to do this by piping: a = b | c
        for key in translation_data_addresses:
            a = translation_data_addresses[key]
            data_addresses[key] = {'ip': a['ip'], 'port': a['port']}

    # print("Get All addresses for:", data_name)
    for key in data_addresses:
        a = data_addresses[key]
        # print("Address:", a['ip'], ';', a['port'])

    return data_addresses  # returns a dict where pubkey is the key, and a small dict containing ip,port as a value


# Watch out, is called with the translation file name
def get_addresses_from_translation_file(data_name):
    trans_file_path = get_trans_file_path(data_name)
    address_list = {}
    if os.path.exists(trans_file_path):
        connection = dc.get_connection(False, trans_file_path)
        cursor = connection.cursor()
        cursor.execute("SELECT pubkey, ip, port FROM address")

        rows = cursor.fetchall()
        for row in rows:
            pubkey = row[0]
            ip = row[1]
            port = row[2]
            address_list[pubkey] = {'ip': ip, 'port': port}

    return address_list


# Checks if the address translation file contain exactly these addesses, else return False
def verify_addres_database(data_name):
    existing_addresses = get_addresses_from_translation_file(data_name)
    new_addresses = get_all_addresses(data_name)

    # Verify there are the same amount of addresses in the trans file
    return existing_addresses == new_addresses


# Reset an address translation file with its addresses, you should update it as the file have changed! (like ./10f.exe -u)
# Note: YOU must update version of the file, this is a raw, no checks update.
def set_addresses(data_name):
    trans_file_path = get_trans_file_path(data_name)
    if os.path.exists(trans_file_path):
        # Get all addresses
        new_addresses = get_all_addresses(data_name)

        # Clean out trans file
        print("Clean out address from", trans_file_path, flush=True)
        if os.path.exists(trans_file_path):
            connection = dc.get_connection(False, trans_file_path)
            if connection:
                connection.execute('''DELETE FROM address''')
                connection.commit()

                # Insert the new_addresses in trans file
                for rsa in new_addresses:
                    ip = new_addresses[rsa]['ip']
                    port = new_addresses[rsa]['port']
                    connection.execute('''INSERT INTO address (pubkey, ip, port) VALUES (?, ?, ?)''', (rsa, ip, port))
                    connection.commit()



