# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

# This module is doing the parallel downloading

import database_connection as dc
from get_data_from_link import extract_data_from_database_link, save_off_addresses


# Note: when a process don't have a free block to download, try to download one in the making (it might be stuck)

# todo: use threads OR processes in the doc when we have chosen one of the two
# Threads: nice to use but is subject to GIL
# Processes: harder to control (stuck processes in windows?), but we'll use the database to do that

def create_download_database(linkfile):
    print('Creating download database:')

    connection = dc.get_connection(checkifexisting=False, database_file_path=linkfile + '_download.db')
    # Table for downloading
    # The block ID, each thread/process will grab one and try to download
    # start + size is the part of the data we want to download
    # done is:
    # -1: not started, so a process can take it
    # 0: started
    # 1: done

    connection.execute('''CREATE TABLE download
             (block INTEGER PRIMARY KEY AUTOINCREMENT,
             start                  INTEGER,
             size                   INTEGER,
             done                   INTEGER
              );''')

    # target_version is to be checked so that we download the correct version
    # found_version is the highest version we found
    # stop = 1 means all threads/processes should just bail out, it happens either when:
    #    found_version > target_version
    #    Download is finished: all 'done' equals 1 in table download
    #    user cancels the download / timeout / ...
    connection.execute('''CREATE TABLE information
             (
             target_version          INTEGER,
             found_version           INTEGER,
             stop                    INTEGER
              );''')

    # A direct copy from the address_list in the link file
    # This is useful as we might add information about the nodes here
    # For example if a node has a stale version, of if it's down
    connection.execute('''CREATE TABLE address_list
             (
              rsa_pub       TEXT,
              ip            TEXT,
              port          TEXT
             );''')

    return connection


# This takes a link and returns the downloaded .aes file (or tar.gz or whatever)
# Caller must process the rest to get the data
# It is supposed to replace parts in get_data_from_database_link()
def download_raw_data(linkfile, dest_folder):
    linkfile = linkfile.replace('\\', '/')
    dest_folder = dest_folder.replace('\\', '/')

    info, address_list = extract_data_from_database_link(linkfile)
    minimum_required_version = info['file_version']
    save_off_addresses(address_list)

    #################################
    # Generate a database
    connection = create_download_database(linkfile)

    #################################
    # Get the size of the data

    #################################
    # Split it up in N blocks
    blocksize = 1000

    #################################
    # Insert them in the database
    cursor = connection.cursor()

    # Truncate tables:
    cursor.execute("DELETE FROM download")
    cursor.execute("DELETE FROM information")
    cursor.execute("DELETE FROM address_list")

    # Loop addresses
    for address in address_list:
        print('Address:', address['ip'], address['port'])
        # add them
        cursor.execute("INSERT INTO address_list ('rsa_pub', 'ip', 'port') VALUES ('30820222300d06092a8...', '127.0.0.1', 2000)")  # todo:

    # Loop blocks
    for b in range(0, 999):
        cursor.execute("INSERT INTO download ('start', 'size', 'done') VALUES (0, 999, -1)")  # todo:

    cursor.execute("INSERT INTO information ('target_version', 'found_version', 'stop') VALUES (7, 0, 0)")  # todo:

    connection.commit()

    #################################
    # Copy over the addresses from the link to the database

    # Get the addresses

    connection.commit()

    #################################
    # Create M threads/processes

    processes = 4  # Try to make it 1 process per block, or divided by some integer rounded up.

    # Start all processes and put them in some container

    #################################
    # Wait until it is all downloaded, and print some info like: [##000#_0#0###0_#] 48%

    # Did it work out, or is the version wrong? If so start over

    #################################
    # Fusion all the blocks into one file

    #################################
    # Clean up all the blocks and the database















