# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

import sys
import os

from Cryptodome.Cipher import AES
from Cryptodome.Util import Counter
from Cryptodome import Random
import binascii
import os.path
import tarfile
from tarfile import GNU_FORMAT  # To remove those pesky @PaxHeader infos in our tar.gz
import re

import database_connection as dc
import database_query as dq

from encryption import CipherHelper
import locator


# This loads and encrypts data, inserts it into the table 'data_owned'
# stores encrypted data in folder '/data/local/' as it is our owned data.
# Later on, the scheduler will ask other nodes for swicharoo (e.g storing our data at their place, and their data at our place in /data/$nodenr$/)
def create_our_data(name, filefolder, file, rsa_pub_key, address_translation_file=False):
    # Insert in database
    print('Insert new data:')
    print('Name:', name)
    print('Folder:', filefolder)
    print('File:', file)

    filefolder = filefolder.replace('\\', '/')
    file = file.replace('\\', '/')

    connection = dc.get_connection()

    filepath = os.path.join(filefolder, file)

    data = None
    with open(filepath, 'rb') as f:
        data = f.read()

    if data is None:
        print('File not found')
        sys.exit(-3)

    cursor = connection.cursor()
    cursor.execute("SELECT KEY, IV FROM data_owned WHERE NAME = '" + name + "'")
    rows = cursor.fetchall()
    # Does a database entry exists?
    if len(rows) > 0:
        print("#Error entry already exist (insert_data) use update_data.py instead")
        sys.exit(0)

    ch = CipherHelper()

    # Encrypt with AES-CTR
    encrypted_data = ch.encrypt(data)

    # Make a 'local' folder if it doesn't already exist
    directory = os.path.join(locator.module_path(), 'data/local/')
    if not os.path.exists(directory):
        os.makedirs(directory)

    # Store the encypted data in the data folder so listener can serve it to clients:
    with open(directory + name + '.aes', 'wb') as f:
        f.write(encrypted_data)

    AEShex = ch.aes_key.hex()
    IVhex = ch.raw_iv.hex()

    at = 1 if address_translation_file else 0
    cursor.execute("INSERT INTO data_owned(NAME, address_translation_file, ORIGINALFILENAME, KEY, IV, PUBKEY) VALUES(?, ?, ?, ?, ?, ?);",
                   (name, at, file, AEShex, IVhex, rsa_pub_key))
    connection.commit()


# Accessible as python function:
def insert_file(data_name, folder, file_name, address_translation_file=False):
    _, pub = dq.get_rsa_hex_keys()
    create_our_data(data_name, folder, file_name, pub, address_translation_file)


# Accessible as python function:
def insert_folder(data_name, folder, regex, silent=False):
    if not silent:
        print('Insert folder:')

    # get all the files in 'folder' and zip it:
    full_data_name = data_name + '.tar.gz'
    with tarfile.open(full_data_name, "w:gz", format=GNU_FORMAT) as tar_handle:
        for root, _, files in os.walk(folder):
            for file in files:
                file_path = os.path.join(root, file)
                if re.search(regex, file_path):
                    if not silent:
                        print('# ADD:', file_path)
                    tar_handle.add(file_path)  #, arcname=os.path.basename(os.path.join(root)))
                else:
                    if not silent:
                        print('# SKIP:', file_path)


        #tar_handle.add(folder)  # arcname=os.path.basename(folder) for relative path?

    _, pub = dq.get_rsa_hex_keys()
    # name, filefolder, file, rsa_pub_key
    create_our_data(data_name, './', full_data_name, pub)
    # delete tar.gz file
    os.remove(full_data_name)

