# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

import time

import rsa_helper as rh

from socket_communication import SecureCommunication
import database_query as dq
from configuration import get_configuration_value


"""
Helper functions for setting and updating information about data stored by other nodes
This is always used after a swicharoo

"""


# Not in use (used by not used function below refresh_data_information())
# requires a 'set'/'update' of the information about the data they store for us: [name, rsa_pub_key, timestamp, Address+], digest (+ == at least one address)
def requre_update_data_information(sc, data_id):
    address_list = dq.get_address_list(data_id)
    prv, pub = dq.get_rsa_hex_keys()
    name, _ = dq.get_link_data_by_dataid(data_id)
    epoch_time = int(time.time())
    query = name + ',' + pub + ',' + epoch_time  # This (in bytes) is what the digest validates! Change here, change in listener too.
    query = bytes(query)
    digest = rh.create_digest(query, prv)

    query = query + b',' + digest + b',' + bytes(address_list)  # Address list is trusted by Node (as we are in a secure connection), User must verify itself...

    sc.send(b'NEW_METADT' + query)  # "new Meta Data"


# Not in use
# Calls all the nodes storing this data, and requires an requre_update_data_information() for each
def refresh_data_information(data_id):
    # Get all the fingers (e.g nodes storing our data)
    fingers = dq.get_all_fingers(data_id)

    # update all the fingers pointing to our data
    for fingerid in fingers:
        nodeid, _ = dq.get_finger_data(fingerid)  # get the node of this finger (one of the fingers pointing on our data)
        if nodeid > 0:
            tmp = dq.get_node_data(nodeid)
            if tmp:
                ip = tmp[0]
                port = tmp[1]
                rsa_pub_key = tmp[2]
                # Make a secure connection to the node holding the finger
                sc = SecureCommunication(ip, port, rsa_pub_key)
                if sc.is_active():
                    requre_update_data_information(sc, data_id)
                    sc.close()

    

    # todo: Update (connection_tries, connection_fails, last_connection_datetime) in db:node accordingly with update_node_quality()

